% MTODO 2
% TEST 2
%
% La referencia utilizada ha sido:
%
% 	J. M. Amig, S. Zambrano, M. A. F. Sanjun
%	DETECTING DETERMINISM IN TIME SERIES WITH ORDINAL PATTERNS: A
%	COMPARATIVE STUDY
%	International Journal of Bifurcation and Chaos, Vol. 20, No. 9 (2010),
%	2915-2924
%
% Damos los siguientes pasos para aplicar el test:
%
% PASO 0: Se introducen los parmetros de entrada de la funcin, que
% son la serie temporal, la dimensin de inmersin y el nivel de confianza.
%
% PASO 1: Clculos previos.
% PASO 2: Calculamos el estadstico de contraste.
% PASO 3: Realizamos el contraste de hiptesis.

function t2=test_2(serie_temp,m,nivel_confianza)

% PASO 1
% CLCULOS PREVIOS

serie=serie_temp; % serie temporal
T=length(serie); % nmero de elementos de la serie
k=fix((T-1)/(m-1)); % nmero de tramos
nivel_significacion=1-nivel_confianza/100; % nivel de significacin

% PASO 2
% CALCULAMOS EL ESTADSTICO DE CONTRASTE 

% La serie se divide en tramos bajo las siguientes condiciones:
%
%   1) El primer tramo comienza en el primer elemento de la serie.
%   2) Todos los tramos tienen la misma longitud, siendo sta la
%   dimensin de inmersin m.
%   3) Los tramos abarcan elementos consecutivos.
%   4) El solapamiento entre dos tramos consecutivos ser de un solo
%   elemento, de modo que el desplazamiento entre ellos es de m-1
%   elementos.
%

% Asignamos una permutacin a cada tramo en funcin del orden de los
% elementos que lo integran.

% Para determinar el estadstico de contraste hay que calcular la
% frecuencia absoluta de todas las posibles permutaciones.

M_permutaciones=zeros(factorial(m),m);
Fabs_permutacion=zeros(factorial(m),1);

trozo_serie1=serie(1:m);
[trozo_ordenado1,permutacion1]=sort(trozo_serie1);
M_permutaciones(1,:)=permutacion1;
Npermutaciones_diferentes=1;
Fabs_permutacion(1)=1;

for cont1=m:m-1:T-m+1
    
    trozo_serie=serie(cont1:cont1+m-1);
    [trozo_ordenado,permutacion]=sort(trozo_serie);
        
    testigo=1;
    
    for cont2=1:Npermutaciones_diferentes
        
        if isequal(M_permutaciones(cont2,:),permutacion)
            
            Fabs_permutacion(cont2)=Fabs_permutacion(cont2)+1;
            testigo=0;
            break            
            
        end
        
    end
    
    if testigo
        
    Npermutaciones_diferentes=Npermutaciones_diferentes+1;    
    M_permutaciones(Npermutaciones_diferentes,:)=permutacion;
    Fabs_permutacion(Npermutaciones_diferentes)=1;
    
    end
    
end

% Despus de determinar la frecuencia absoluta de cada permutacin ya se
% puede calcular el estadstico de contraste.
% El estadstico de contraste es:
%
%       ESTADISTICO_CONTRASTE=(m!/k)*SUM(fa_pi^2)-k
%
% donde fa_pi denota la frecuencia absoluta de cada permutacin. 
% Observamos que el estadstico de contraste depende de la dimensin de
% inmersin m, del nmero de ventanas deslizantes k y de la frecuencia
% absoluta de cada permutacin fa_pi.

suma_auxiliar=0; 

for cont3=1:Npermutaciones_diferentes
      
    suma_auxiliar=suma_auxiliar+Fabs_permutacion(cont3)^2;
          
end

Estadistico_contraste=(factorial(m)/k)*suma_auxiliar-k; % estadstico de contraste

% PASO 3
% REALIZAMOS EL CONTRASTE DE HIPTESIS

grados_libertad=factorial(m)-1; % grados de libertad
chi_cuadrado=chi2inv(1-nivel_significacion,grados_libertad); % valor crtico

% No rechazamos la hiptesis nula cuando el estadstico de contraste es
% menor o igual al valor crtico; caso en el que asignamos a la variable t2
% el valor unidad.
% En otro caso la hiptesis nula se rechaza, y t2 adquiere el valor nulo.

if Estadistico_contraste<=chi_cuadrado
    
    t2=1;
    
else
        
    t2=0;
        
end
