% MTODO 3 
% TEST 3-3
%
% La referencia utilizada ha sido:
%
%	J. S. Cnovas, A. Guillamn
%	PERMUTATIONS AND TIME SERIES ANALYSIS
%	Chaos 19 (2009), 043103
%
% Aplicamos el test siguiendo los pasos siguientes:
%
% PASO 0: Se introducen los parmetros de entrada de la funcin, que
% son la serie temporal, la dimensin de inmersin y el nivel de confianza.
% 
% PASO 1: Clculos previos.
% PASO 2: Calculamos el estadstico de contraste.
% PASO 3: Realizamos el contraste de hiptesis.

function t3_3=test_3_3(serie_temp,m,nivel_confianza)

% PASO 1
% CLCULOS PREVIOS

serie=serie_temp; % serie temporal
T=length(serie); % nmero de elementos de la serie
nivel_significacion=1-nivel_confianza/100; % nivel de significacin

% PASO 2
% CALCULAMOS EL ESTADSTICO DE CONTRASTE

% Dividimos la serie en varias ventanas.
% La primer ventana empieza en el primer elemento de la serie.
% Entre las ventanas no habr solapamientos, ni tampoco quedarn elementos
% sueltos.
% Las ventanas sern de tamao m*m!.
% Si slo se pudiera tomar una ventana, sta englobara toda la serie
% temporal; as se evita una gran prdida de elementos.

% Si hubiera elementos sobrantes por no ser exacta la divisin T/Tventana,
% dichos elementos sern los del final de la serie.

%if(T>=m*factorial(m)*2)
%    
%    Tventana=m*factorial(m);
%    Nventanas=fix(T/Tventana);
%    
%else
%    
%    Tventana=T;
%    Nventanas=1;
%    
%end

Tventana=12;
Nventanas=4;

% Cada una de las ventanas que conforman la serie temporal se subdivide, a
% su vez, en varios tramos. Estos cumplen lo siquiente:
% 
%   1) El primer tramo empieza en el primer elemento de la ventana.
%   2) Los tramos son de igual amplitud, y sta viene fijada por el valor
%   de la dimensin de inmersin m.
%   3) Los elementos abarcados por los tramos son elementos consecutivos.
%   4) No hay solapamiento entre tramos consecutivos, siendo el
%   desplazamiento de un tramo al siguiente de m elementos.
%

% Asociamos a cada tramo una permutacin. Dicha permutacin informa del
% orden de los elementos que componen el tramo.

% A continuacin se cuenta el nmero de permutaciones diferentes presentes
% en cada una de las ventanas en que se ha dividido la serie.

k=Tventana/m; % nmero de tramos en que se divide cada ventana
Nmax_permutaciones_diferentes=min(fix(k),factorial(m)); % nmero mximo de permutaciones diferentes en cada ventana

% Construimos un vector para almacenar en sus componentes el nmero de
% permutaciones distintas encontradas en cada ventana.

v_Npermutaciones_diferentes=zeros(Nventanas,1);

for cont1=1:Nventanas
    
    ventana=serie((cont1-1)*Tventana+1:cont1*Tventana);
    M_permutaciones=zeros(factorial(m),m);

    trozo_ventana1=ventana(1:m);
    [trozo_ordenado1,permutacion1]=sort(trozo_ventana1);
    M_permutaciones(1,:)=permutacion1;
    Npermutaciones_diferentes=1;
    
    for cont2=m+1:m:Tventana-m+1
    
        trozo_ventana=ventana(cont2:cont2+m-1);
        [trozo_ordenado,permutacion]=sort(trozo_ventana);
        
        testigo=1;
    
        for cont3=1:Npermutaciones_diferentes
        
            if isequal(M_permutaciones(cont3,:),permutacion)
            
                testigo=0;
                break            
            
            end
        
        end
    
        if testigo
        
        Npermutaciones_diferentes=Npermutaciones_diferentes+1;    
        M_permutaciones(Npermutaciones_diferentes,:)=permutacion;
    
        end
    
    end

    v_Npermutaciones_diferentes(cont1)=Npermutaciones_diferentes;

end

% Determinamos la funcin de distribucin terica FW.

% Para calcular esta funcin hay que recurrir a la funcin de probabilidad.
% Los posibles valores que puede adquirir la funcin de probabilidad estn
% almacenados en un archivo, del que nicamente hay que ir leyendo dichos
% valores.

probabilidad=load(strcat('VPROB_T',num2str(Tventana),'m',num2str(m),'.txt'));
probabilidad=probabilidad';

% La funcin de distribucin se calcula del modo siguiente:
%
%       F_W(t)=Pr(W<=t)
%
% donde W es la variable aleatoria que indica el nmero de permutaciones
% diferentes que pueden darse.

suma_FW=0;
FW=zeros(Nmax_permutaciones_diferentes,1); % funcin de distribucin terica

for cont4=1:Nmax_permutaciones_diferentes
    
    suma_FW=suma_FW+probabilidad(cont4);
    FW(cont4)=suma_FW;
    
end

% Determinamos la funcin de distribucin emprica Fn.

% La expresin de clculo de la funcin de distribucin emprica es:
%
%       F_n(t)=j/n, 
% 
% donde j es el nmero de ventanas que presentan un nmero de permutaciones
% diferentes menor o igual a t, y n es el nmero total de ventanas.

auxiliar_Fn=zeros(Nmax_permutaciones_diferentes,1);
Fn=zeros(Nmax_permutaciones_diferentes,1); % funcin de distribucin emprica

for cont5=1:Nventanas
    
    posicion=v_Npermutaciones_diferentes(cont5);
    auxiliar_Fn(posicion)=auxiliar_Fn(posicion)+1/Nventanas;
    
end

Fn(1)=auxiliar_Fn(1);

for cont6=2:Nmax_permutaciones_diferentes
    
    Fn(cont6)=Fn(cont6-1)+auxiliar_Fn(cont6);
    
end

% Calculamos el estadstico de contraste.
% El estadstico de contraste es:
%
%       ESTADSTICO_CONTRASTE=sqrt(n)*D_n
%
% donde n es el nmero total de ventanas, y D_n=max{abs(F_n-F_W)}.

Dn=max(abs(Fn-FW)); % distancia mxima entre Fn y FW
Estadistico_contraste=sqrt(Nventanas)*Dn; % estadstico de contraste

% PASO 3
% REALIZAMOS EL CONTRASTE DE HIPTESIS

% Para realizar el contraste de hiptesis vamos a utilizar el p-valor.
% El p-valor se calcula con la funcin de distribucin continua de la
% distribucin de Kolmogorov, y depende del valor del estadstico de
% contraste.

suma_pValor=0;

for cont7=1:10
    
    suma_pValor=suma_pValor+(-1)^(cont7+1)*exp(-2*(cont7*Estadistico_contraste)^2);
    
end

pValor=2*suma_pValor; % p-valor

% Si el nivel de significacin del test es menor o igual al p-valor, no se
% rechaza la hiptesis nula, y la variable t3_3 toma el valor 1.
% En otro caso se rechaza la hiptesis nula y t3_3 toma el valor 0.

if nivel_significacion<=pValor
    
    t3_3=1;
    
else
    
    t3_3=0;
    
end

v_Npermutaciones_diferentes
Estadistico_contraste
pValor